@@ -29,7 +29,7 @@ target_metadata = models.BASE.metadata
 # ... etc.
 
 
-def run_migrations_online():
+def run_migrations_online() -> None:
     """Run migrations in 'online' mode.
 
     In this scenario we need to create an Engine
