@@ -34,11 +34,11 @@ branch_labels = None
 depends_on = None
 
 
-def upgrade():
+def upgrade() -> None:
     with op.batch_alter_table("envs") as batch_op:
         batch_op.add_column(
             sa.Column("config", sa_types.MutableJSONEncodedDict))
 
 
-def downgrade():
+def downgrade() -> None:
     raise exceptions.DowngradeNotSupported()
