@@ -41,7 +41,7 @@ deployments_helper = sa.Table(
 )
 
 
-def upgrade():
+def upgrade() -> None:
     connection = op.get_bind()
     inspector = sa.inspect(connection)
     if "deployments" not in inspector.get_table_names():
@@ -67,5 +67,5 @@ def upgrade():
                     config=config))
 
 
-def downgrade():
+def downgrade() -> None:
     raise exceptions.DowngradeNotSupported()
