@@ -16,7 +16,6 @@
 import collections
 import copy
 import datetime as dt
-import six
 import uuid
 
 from rally.common import db
@@ -324,11 +323,12 @@ class Task(object):
 
     def to_dict(self):
         db_task = self.task
-        env_name = db.env_get(self.task["env_uuid"])["name"]
-        db_task["env_name"] = env_name
-        db_task["deployment_name"] = env_name
-        db_task["deployment_uuid"] = db_task["env_uuid"]
-        self._serialize_dt(db_task)
+        if self.task.get("env_uuid"):
+            env_name = db.env_get(self.task["env_uuid"])["name"]
+            db_task["env_name"] = env_name
+            db_task["deployment_name"] = env_name
+            db_task["deployment_uuid"] = db_task["env_uuid"]
+            self._serialize_dt(db_task)
         for subtask in db_task.get("subtasks", []):
             self._serialize_dt(subtask)
             for workload in subtask["workloads"]:
@@ -344,9 +344,9 @@ class Task(object):
         return db.task_get_status(uuid)
 
     @staticmethod
-    def list(status=None, deployment=None, tags=None):
+    def list(status=None, deployment=None, tags=None, uuids_only=False):
         return [Task(db_task) for db_task in db.task_list(
-            status, env=deployment, tags=tags)]
+            status, env=deployment, tags=tags, uuids_only=False)]
 
     @staticmethod
     def delete_by_uuid(uuid, status=None):
@@ -450,7 +450,7 @@ class Task(object):
                 actions_list.extend(action["children"])
 
         for e in result["error"]:
-            if not isinstance(e, (six.string_types, six.text_type)):
+            if not isinstance(e, str):
                 LOG.warning("error value has wrong type '%s', should be 'str'"
                             % type(e))
                 return False
@@ -559,7 +559,7 @@ class Workload(object):
         task["subtasks"] = [collections.OrderedDict()]
         subtask = task["subtasks"][0]
         subtask["title"] = workload["name"]
-        subtask["description"] = workload["description"]
+        subtask["description"] = workload.get("description", "")
         subtask["scenario"] = {workload["name"]: workload["args"]}
         subtask["contexts"] = workload["contexts"]
         subtask["runner"] = {workload["runner_type"]: workload["runner"]}
