@@ -14,15 +14,15 @@
 #    under the License.
 
 import abc
+import contextlib
+import itertools
 import math
+import os
 
-import six
-
-from rally.task.processing import utils
+from rally.common import utils as cutils
 
 
-@six.add_metaclass(abc.ABCMeta)
-class StreamingAlgorithm(object):
+class StreamingAlgorithm(object, metaclass=abc.ABCMeta):
     """Base class for streaming computations that scale."""
 
     @abc.abstractmethod
@@ -100,9 +100,9 @@ class StdDevComputation(StreamingAlgorithm):
         self.mean = self.mean_computation.result()
         self.count += other.count
 
-        self.dev_sum = (dev_sum1 + count1 * mean1 ** 2 +
-                        dev_sum2 + count2 * mean2 ** 2 -
-                        self.count * self.mean ** 2)
+        self.dev_sum = (dev_sum1 + count1 * mean1 ** 2
+                        + dev_sum2 + count2 * mean2 ** 2
+                        - self.count * self.mean ** 2)
 
     def result(self):
         # NOTE(amaretskiy): Need at least two values to be processed
@@ -151,34 +151,69 @@ class MaxComputation(StreamingAlgorithm):
         return self._value
 
 
-class PercentileComputation(StreamingAlgorithm):
-    """Compute percentile value from a stream of numbers."""
+class PointsSaver(StreamingAlgorithm):
 
-    def __init__(self, percent, length):
-        """Init streaming computation.
+    def __init__(self, chunk_size=10000, sep=" "):
+        self.chunk_size = chunk_size
+        self._sep = sep
+        self._filename = cutils.generate_random_path()
+        self._chunk = []
+        self._current_chunk_size = 0
+        self._deleted = False
 
-        :param percent: numeric percent (from 0.00..1 to 0.999..)
-        :param length: count of the measurements
-        """
-        if not 0 < percent < 1:
-            raise ValueError("Unexpected percent: %s" % percent)
-        self._percent = percent
-
-        self._graph_zipper = utils.GraphZipper(length, 10000)
+    def _dump_chunk(self):
+        with open(self._filename, "a") as f:
+            f.write(
+                " ".join(
+                    itertools.chain(
+                        (" ", ),
+                        map(lambda x: str(x), self._chunk))
+                )
+            )
+        self._chunk = []
+        self._current_chunk_size = 0
 
     def add(self, value):
-        self._graph_zipper.add_point(value)
+        if self._deleted:
+            raise TypeError("Cannot add more points since %s is in deleted "
+                            "state." % self.__class__.__name__)
+
+        self._chunk.append(value)
+        self._current_chunk_size += 1
+        if self._current_chunk_size >= self.chunk_size:
+            self._dump_chunk()
 
     def merge(self, other):
-        # TODO(ikhudoshyn): Implement me
-        raise NotImplementedError()
+        if self._deleted:
+            raise TypeError("Cannot merge points since %s is in deleted "
+                            "state." % self.__class__.__name__)
+        for point in other.result():
+            self.add(point)
 
     def result(self):
-        results = list(
-            map(lambda x: x[1], self._graph_zipper.get_zipped_graph()))
-        if results:
-            return utils.percentile(results, self._percent)
-        return None
+        if self._deleted:
+            raise TypeError("Cannot fetch points since %s is in deleted "
+                            "state." % self.__class__.__name__)
+
+        if os.path.isfile(self._filename):
+            with open(self._filename) as f:
+                data = f.read().strip(self._sep)
+            res = [float(p) for p in data.split(self._sep) if p]
+        else:
+            # the number of points were less than self.chunk_size, so they were
+            #   not saved to the disk. OR no points at all
+            res = []
+
+        if self._chunk:
+            res.extend(self._chunk)
+        return res
+
+    def reset(self):
+        with contextlib.suppress(FileNotFoundError):
+            os.remove(self._filename)
+        self._deleted = True
+        self._chunk = []
+        self._current_chunk_size = 0
 
 
 class IncrementComputation(StreamingAlgorithm):
@@ -195,3 +230,36 @@ class IncrementComputation(StreamingAlgorithm):
 
     def result(self):
         return self._count
+
+
+class DegradationComputation(StreamingAlgorithm):
+    """Calculates degradation from a stream of numbers
+
+    Finds min and max values from a stream and then calculates
+    ratio between them in percentage. Works only with positive numbers.
+    """
+
+    def __init__(self):
+        self.min_value = MinComputation()
+        self.max_value = MaxComputation()
+
+    def add(self, value):
+        if value <= 0.0:
+            raise ValueError("Unexpected value: %s" % value)
+        self.min_value.add(value)
+        self.max_value.add(value)
+
+    def merge(self, other):
+        min_result = other.min_value.result()
+        if min_result is not None:
+            self.min_value.add(min_result)
+        max_result = other.max_value.result()
+        if max_result is not None:
+            self.max_value.add(max_result)
+
+    def result(self):
+        min_result = self.min_value.result()
+        max_result = self.max_value.result()
+        if min_result is None or max_result is None:
+            return 0.0
+        return (max_result / min_result - 1) * 100.0
