@@ -13,11 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
+
 import collections
 import multiprocessing
 import queue as Queue
 import threading
 import time
+import typing as t
 
 from rally.common import logging
 from rally.common import utils
@@ -25,12 +28,29 @@ from rally.common import validation
 from rally import consts
 from rally.task import runner
 
+if t.TYPE_CHECKING:  # pragma: no cover
+    from rally.task import scenario
+
 LOG = logging.getLogger(__name__)
 
 
-def _worker_process(queue, iteration_gen, timeout, times, max_concurrent,
-                    context, cls, method_name, args, event_queue, aborted,
-                    runs_per_second, rps_cfg, processes_to_start, info):
+def _worker_process(
+    queue: multiprocessing.Queue[runner.ScenarioRunnerResult],
+    iteration_gen: t.Iterator[int],
+    timeout: float | None,
+    times: int,
+    max_concurrent: int,
+    context: dict[str, t.Any],
+    cls: type[scenario.Scenario],
+    method_name: str,
+    args: dict[str, t.Any],
+    event_queue: multiprocessing.Queue[dict[str, t.Any]],
+    aborted: multiprocessing.synchronize.Event,
+    runs_per_second: t.Callable[[dict[str, float] | float, float, int], float],
+    rps_cfg: dict[str, float] | float,
+    processes_to_start: int,
+    info: dict[str, t.Any]
+) -> None:
     """Start scenario within threads.
 
     Spawn N threads per second. Each thread runs the scenario once, and appends
@@ -55,7 +75,7 @@ def _worker_process(queue, iteration_gen, timeout, times, max_concurrent,
     :param info: info about all processes count and counter of runned process
     """
 
-    pool = collections.deque()
+    pool: collections.deque[threading.Thread] = collections.deque()
     if isinstance(rps_cfg, dict):
         rps = rps_cfg["start"]
     else:
@@ -69,7 +89,9 @@ def _worker_process(queue, iteration_gen, timeout, times, max_concurrent,
         (sleep * info["processes_counter"]) / info["processes_to_start"])
 
     start = time.time()
-    timeout_queue = Queue.Queue()
+    timeout_queue: Queue.Queue[
+        tuple[threading.Thread, float] | tuple[None, None]
+    ] = Queue.Queue()
 
     if timeout:
         collector_thr_by_timeout = threading.Thread(
@@ -124,14 +146,24 @@ def _worker_process(queue, iteration_gen, timeout, times, max_concurrent,
 class CheckPRSValidator(validation.Validator):
     """Additional schema validation for rps runner"""
 
-    def validate(self, context, config, plugin_cls, plugin_cfg):
-        if isinstance(plugin_cfg["rps"], dict):
+    def validate(
+        self,
+        context: dict[str, t.Any],
+        config: dict[str, t.Any] | None,
+        plugin_cls: type[runner.plugin.Plugin],
+        plugin_cfg: dict[str, t.Any] | None
+    ) -> None:
+        if plugin_cfg and isinstance(plugin_cfg["rps"], dict):
             if plugin_cfg["rps"]["end"] < plugin_cfg["rps"]["start"]:
                 msg = "rps end value must not be less than rps start value."
-                return self.fail(msg)
+                self.fail(msg)
 
 
-def _runs_per_second(rps_cfg, start_timer, number_of_processes):
+def _runs_per_second(
+    rps_cfg: dict[str, float] | float,
+    start_timer: float,
+    number_of_processes: int
+) -> float:
     """At the given second return desired rps."""
 
     if not isinstance(rps_cfg, dict):
@@ -219,7 +251,13 @@ class RPSScenarioRunner(runner.ScenarioRunner):
         "additionalProperties": False
     }
 
-    def _run_scenario(self, cls, method_name, context, args):
+    def _run_scenario(
+        self,
+        cls: type[scenario.Scenario],
+        method_name: t.Literal["run"],
+        context: dict[str, t.Any],
+        args: dict[str, t.Any]
+    ) -> None:
         """Runs the specified scenario with given arguments.
 
         Every single scenario iteration is executed with specified
@@ -261,10 +299,14 @@ class RPSScenarioRunner(runner.ScenarioRunner):
                              concurrency_per_worker=concurrency_per_worker,
                              concurrency_overhead=concurrency_overhead)
 
-        result_queue = multiprocessing.Queue()
-        event_queue = multiprocessing.Queue()
+        result_queue: multiprocessing.Queue[runner.ScenarioRunnerResult] = (
+            multiprocessing.Queue())
+        event_queue: multiprocessing.Queue[dict[str, t.Any]] = (
+            multiprocessing.Queue())
 
-        def worker_args_gen(times_overhead, concurrency_overhead):
+        def worker_args_gen(
+            times_overhead: int, concurrency_overhead: int
+        ) -> t.Generator[tuple[t.Any, ...], None, None]:
             """Generate arguments for process worker.
 
             Remainder of threads per process division is distributed to
