@@ -32,7 +32,8 @@ class Chart(plugin.Plugin, metaclass=abc.ABCMeta):
     prepare data that is suitable for rendering by JavaScript.
     """
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def widget(self):
         """Widget name to display this chart by JavaScript."""
 
@@ -315,7 +316,8 @@ class Table(Chart, metaclass=abc.ABCMeta):
     widget = "Table"
     _styles = {}
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def columns(self):
         """List of columns names."""
 
@@ -450,21 +452,25 @@ class MainStatsTable(Table):
                 data["<no-name-action>"] = {"duration": 0, "count": 1,
                                             "failed": True, "children": {}}
         total_duration = iteration["duration"] + iteration["idle_duration"]
-        data["total"] = {"duration": total_duration,
-                         "count": 1,
-                         "failed": bool(iteration["error"]),
-                         "children": collections.OrderedDict(
-                             [("duration", {
-                                 "duration": iteration["duration"],
-                                 "count": 1,
-                                 "failed": bool(iteration["error"]),
-                                 "children": []}),
-                              ("idle_duration", {
-                                  "duration": iteration["idle_duration"],
-                                  "count": 1,
-                                  "failed": bool(iteration["error"]),
-                                  "children": []})
-                              ])}
+        data["total"] = {
+            "duration": total_duration,
+            "count": 1,
+            "failed": bool(iteration["error"]),
+            "children": dict(
+                duration={
+                    "duration": iteration["duration"],
+                    "count": 1,
+                    "failed": bool(iteration["error"]),
+                    "children": {}
+                },
+                idle_duration={
+                    "duration": iteration["idle_duration"],
+                    "count": 1,
+                    "failed": bool(iteration["error"]),
+                    "children": {}
+                }
+            )
+        }
 
         self._add_data(data)
 
