@@ -13,8 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
+
 import copy
 import random
+import typing as t
 
 from rally.common import cfg
 from rally.common import logging
@@ -26,6 +29,9 @@ from rally.task import atomic
 from rally.task import functional
 from rally.task.processing import charts
 
+if t.TYPE_CHECKING:  # pragma: no cover
+    S = t.TypeVar("S", bound="Scenario")
+
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -41,7 +47,11 @@ CONF_OPTS = [
 CONF.register_opts(CONF_OPTS)
 
 
-def configure(name, platform="default", context=None):
+def configure(
+    name: str,
+    platform: str = "default",
+    context: dict[str, t.Any] | None = None
+) -> t.Callable[[type[S]], type[S]]:
     """Configure scenario by setting proper meta data.
 
     This can also transform plain function into scenario plugin, however
@@ -54,15 +64,15 @@ def configure(name, platform="default", context=None):
                     If there are custom user specified contexts this one
                     will be updated by provided contexts.
     """
-    context = context or {}
+    context_dict: dict[str, t.Any] = context or {}
 
-    def wrapper(cls):
+    def wrapper(cls: type[S]) -> type[S]:
         # TODO(boris-42): Drop this check as soon as we refactor rally report
         if "." not in name.strip("."):
             raise exceptions.RallyException(
                 "Scenario name must include a dot: '%s'" % name)
 
-        for c in context:
+        for c in context_dict:
             if "@" not in c:
                 msg = ("Old fashion plugin configuration detected in "
                        " `%(scenario)s' scenario. Use full name for "
@@ -75,12 +85,17 @@ def configure(name, platform="default", context=None):
         cls = plugin.configure(name=name, platform=platform)(cls)
         cls._meta_setdefault("default_context", {})
 
-        cls._meta_get("default_context").update(context)
+        cls._meta_get("default_context").update(context_dict)
         return cls
 
     return wrapper
 
 
+class _Output(t.TypedDict):
+    additive: list[dict[str, t.Any]]
+    complete: list[dict[str, t.Any]]
+
+
 @validation.add_default("args-spec")
 @plugin.base()
 class Scenario(plugin.Plugin,
@@ -95,27 +110,32 @@ class Scenario(plugin.Plugin,
     RESOURCE_NAME_FORMAT = "s_rally_XXXXXXXX_XXXXXXXX"
 
     @classmethod
-    def _get_resource_name_format(cls):
+    def _get_resource_name_format(cls) -> str:
         return (CONF.scenario_resource_name_format
                 or super(Scenario, cls)._get_resource_name_format())
 
-    def __init__(self, context=None):
+    def __init__(self, context: dict[str, t.Any] | None = None) -> None:
         super(Scenario, self).__init__()
         self.context = context or {}
         self.task = self.context.get("task", {})
         self._idle_duration = 0.0
-        self._output = {"additive": [], "complete": []}
+        self._output: _Output = dict(additive=[], complete=[])
 
-    def get_owner_id(self):
+    def get_owner_id(self) -> str | None:
         if "owner_id" in self.context:
             return self.context["owner_id"]
         return super(Scenario, self).get_owner_id()
 
     @classmethod
-    def get_default_context(cls):
+    def get_default_context(cls) -> dict[str, t.Any]:
         return copy.deepcopy(cls._meta_get("default_context"))
 
-    def sleep_between(self, min_sleep, max_sleep=None, atomic_delay=0.1):
+    def sleep_between(
+        self,
+        min_sleep: float,
+        max_sleep: float | None = None,
+        atomic_delay: float = 0.1
+    ) -> None:
         """Call an interruptable_sleep() for a random amount of seconds.
 
         The exact time is chosen uniformly randomly from the interval
@@ -138,11 +158,15 @@ class Scenario(plugin.Plugin,
         utils.interruptable_sleep(sleep_time, atomic_delay)
         self._idle_duration += sleep_time
 
-    def idle_duration(self):
+    def idle_duration(self) -> float:
         """Returns duration of all sleep_between."""
         return self._idle_duration
 
-    def add_output(self, additive=None, complete=None):
+    def add_output(
+        self,
+        additive: dict[str, t.Any] | None = None,
+        complete: dict[str, t.Any] | None = None
+    ) -> None:
         """Add iteration's custom output data.
 
         This saves custom output data to task results. The main way to get
@@ -182,8 +206,24 @@ class Scenario(plugin.Plugin,
                 message = charts.validate_output(key, value)
                 if message:
                     raise exceptions.RallyException(message)
-                self._output[key].append(value)
+                self._output[
+                    key  # type: ignore[literal-required]
+                ].append(value)
+
+    if not t.TYPE_CHECKING:
+        def run(self, **kwargs: t.Any) -> None:
+            """Execute the scenario's workload.
+
+            This method must be implemented by all scenario plugins.
+            It defines the actual workload that the scenario will execute.
+
+            :param kwargs: Scenario-specific arguments from task configuration
+            """
+            raise NotImplementedError()
+    else:
+        run: t.Callable
 
     @classmethod
-    def _get_doc(cls):
-        return cls.run.__doc__
+    def _get_doc(cls) -> str:
+        """Get scenario documentation from run method."""
+        return cls.run.__doc__ or ""
