@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import collections
+from unittest import mock
 
-from rally.common import costilius
 from rally.task import atomic
 from tests.unit import test
 
@@ -26,6 +26,16 @@ class ActionTimerMixinTestCase(test.TestCase):
         inst = atomic.ActionTimerMixin()
         self.assertEqual(inst._atomic_actions, inst.atomic_actions())
 
+    def test_reset_atomic_actions(self):
+        inst = atomic.ActionTimerMixin()
+
+        with atomic.ActionTimer(inst, "test"):
+            pass
+        self.assertNotEqual([], inst.atomic_actions())
+
+        inst.reset_atomic_actions()
+        self.assertEqual([], inst.atomic_actions())
+
 
 class AtomicActionTestCase(test.TestCase):
 
@@ -38,8 +48,17 @@ class AtomicActionTestCase(test.TestCase):
                 with atomic.ActionTimer(inst, "some"):
                     pass
 
-        expected = [("test", 20), ("test (2)", 12), ("some", 4)]
-        self.assertEqual(costilius.OrderedDict(expected),
+        expected = [{"name": "test",
+                     "started_at": 1,
+                     "finished_at": 21,
+                     "children": [{"name": "test",
+                                   "started_at": 3,
+                                   "finished_at": 15,
+                                   "children": [{"name": "some",
+                                                 "started_at": 6,
+                                                 "finished_at": 10,
+                                                 "children": []}]}]}]
+        self.assertEqual(expected,
                          inst.atomic_actions())
 
     @mock.patch("time.time", side_effect=[1, 3])
@@ -55,8 +74,8 @@ class AtomicActionTestCase(test.TestCase):
         except TestException:
             pass
 
-        expected = [("test", 2)]
-        self.assertEqual(costilius.OrderedDict(expected),
+        self.assertEqual([{"name": "test", "children": [], "failed": True,
+                           "started_at": 1, "finished_at": 3}],
                          inst.atomic_actions())
 
     @mock.patch("time.time", side_effect=[1, 3])
@@ -70,7 +89,8 @@ class AtomicActionTestCase(test.TestCase):
 
         inst = Some()
         self.assertEqual(5, inst.some_func(2, 3))
-        self.assertEqual(costilius.OrderedDict({"some": 2}),
+        self.assertEqual([{"name": "some", "children": [],
+                           "started_at": 1, "finished_at": 3}],
                          inst.atomic_actions())
 
     @mock.patch("time.time", side_effect=[1, 3])
@@ -87,39 +107,27 @@ class AtomicActionTestCase(test.TestCase):
 
         inst = TestTimer()
         self.assertRaises(TestException, inst.some_func)
-        self.assertEqual(costilius.OrderedDict({"test": 2}),
+        self.assertEqual([{"name": "test", "children": [], "failed": True,
+                           "started_at": 1, "finished_at": 3}],
                          inst.atomic_actions())
 
-    @mock.patch("time.time", side_effect=[1, 3, 1, 3])
-    def test_optional_action_timer_decorator(self, mock_time):
-
-        class TestAtomicTimer(atomic.ActionTimerMixin):
-
-            @atomic.optional_action_timer("some")
-            def some_func(self, a, b):
-                return a + b
-
-            @atomic.optional_action_timer("some", argument_name="foo",
-                                          default=False)
-            def other_func(self, a, b):
-                return a + b
-
-        inst = TestAtomicTimer()
-        self.assertEqual(5, inst.some_func(2, 3))
-        self.assertEqual(costilius.OrderedDict({"some": 2}),
-                         inst.atomic_actions())
-
-        inst = TestAtomicTimer()
-        self.assertEqual(5, inst.some_func(2, 3, atomic_action=False))
-        self.assertEqual(costilius.OrderedDict(),
-                         inst.atomic_actions())
-
-        inst = TestAtomicTimer()
-        self.assertEqual(5, inst.other_func(2, 3))
-        self.assertEqual(costilius.OrderedDict(),
-                         inst.atomic_actions())
-
-        inst = TestAtomicTimer()
-        self.assertEqual(5, inst.other_func(2, 3, foo=True))
-        self.assertEqual(costilius.OrderedDict({"some": 2}),
-                         inst.atomic_actions())
+    def test_merge_atomic_actions(self):
+        expected = [("foo", {"duration": 2, "count": 1,
+                             "children": collections.OrderedDict()}),
+                    ("bar", {"duration": 5, "count": 2,
+                             "children": collections.OrderedDict()}),
+                    ("do_something_bad", {
+                        "duration": 1, "count": 1, "failed": True,
+                        "children": collections.OrderedDict()})]
+        result = atomic.merge_atomic_actions(
+            [{"name": "foo", "started_at": 4, "finished_at": 6,
+              "children": []},
+             {"name": "bar", "started_at": 6, "finished_at": 8,
+              "children": []},
+             {"name": "bar", "started_at": 8, "finished_at": 11,
+              "children": []},
+             {"name": "do_something_bad", "started_at": 11, "finished_at": 12,
+              "children": [], "failed": True}
+             ])
+        result = list(result.items())
+        self.assertEqual(expected, result)
